@@ -25,16 +25,18 @@
 
 import queue
 from collections import defaultdict
-from typing import Sequence, Tuple, Optional, Dict, TYPE_CHECKING, Set
+from typing import Sequence, Tuple, Optional, Dict, TYPE_CHECKING, Set, Callable
 import time
+import threading
 from threading import RLock
-import attr
 from math import inf
 
-from .util import profiler, with_lock, bh2u
+import attr
+
+from .util import profiler, with_lock
 from .logging import Logger
 from .lnutil import (NUM_MAX_EDGES_IN_PAYMENT_PATH, ShortChannelID, LnFeatures,
-                     NBLOCK_CLTV_EXPIRY_TOO_FAR_INTO_FUTURE)
+                     NBLOCK_CLTV_DELTA_TOO_FAR_INTO_FUTURE, PaymentFeeBudget)
 from .channel_db import ChannelDB, Policy, NodeInfo
 
 if TYPE_CHECKING:
@@ -42,7 +44,6 @@ if TYPE_CHECKING:
 
 DEFAULT_PENALTY_BASE_MSAT = 500  # how much base fee we apply for unknown sending capability of a channel
 DEFAULT_PENALTY_PROPORTIONAL_MILLIONTH = 100  # how much relative fee we apply for unknown sending capability of a channel
-BLACKLIST_DURATION = 3600  # how long (in seconds) a channel remains blacklisted
 HINT_DURATION = 3600  # how long (in seconds) a liquidity hint remains valid
 
 
@@ -73,10 +74,10 @@ class PathEdge:
 
 @attr.s
 class RouteEdge(PathEdge):
-    fee_base_msat = attr.ib(type=int, kw_only=True)
-    fee_proportional_millionths = attr.ib(type=int, kw_only=True)
-    cltv_expiry_delta = attr.ib(type=int, kw_only=True)
-    node_features = attr.ib(type=int, kw_only=True, repr=lambda val: str(int(val)))  # note: for end node!
+    fee_base_msat = attr.ib(type=int, kw_only=True)                # for start_node
+    fee_proportional_millionths = attr.ib(type=int, kw_only=True)  # for start_node
+    cltv_delta = attr.ib(type=int, kw_only=True)                   # for start_node
+    node_features = attr.ib(type=int, kw_only=True, repr=lambda val: str(int(val)))  # note: for end_node!
 
     def fee_for_edge(self, amount_msat: int) -> int:
         return fee_for_edge_msat(forwarded_amount_msat=amount_msat,
@@ -87,7 +88,7 @@ class RouteEdge(PathEdge):
     def from_channel_policy(
             cls,
             *,
-            channel_policy: 'Policy',
+            channel_policy: 'Policy',  # for start_node
             short_channel_id: bytes,
             start_node: bytes,
             end_node: bytes,
@@ -102,19 +103,9 @@ class RouteEdge(PathEdge):
             short_channel_id=ShortChannelID.normalize(short_channel_id),
             fee_base_msat=channel_policy.fee_base_msat,
             fee_proportional_millionths=channel_policy.fee_proportional_millionths,
-            cltv_expiry_delta=channel_policy.cltv_expiry_delta,
+            cltv_delta=channel_policy.cltv_delta,
             node_features=node_info.features if node_info else 0)
 
-    def is_sane_to_use(self, amount_msat: int) -> bool:
-        # TODO revise ad-hoc heuristics
-        # cltv cannot be more than 2 weeks
-        if self.cltv_expiry_delta > 14 * 144:
-            return False
-        total_fee = self.fee_for_edge(amount_msat)
-        if not is_fee_sane(total_fee, payment_amount_msat=amount_msat):
-            return False
-        return True
-
     def has_feature_varonion(self) -> bool:
         features = LnFeatures(self.node_features)
         return features.supports(LnFeatures.VAR_ONION_OPT)
@@ -124,7 +115,7 @@ class RouteEdge(PathEdge):
 
 @attr.s
 class TrampolineEdge(RouteEdge):
-    invoice_routing_info = attr.ib(type=bytes, default=None)
+    invoice_routing_info = attr.ib(type=Sequence[bytes], default=None)
     invoice_features = attr.ib(type=int, default=None)
     # this is re-defined from parent just to specify a default value:
     short_channel_id = attr.ib(default=ShortChannelID(8), repr=lambda val: str(val))
@@ -135,42 +126,42 @@ class TrampolineEdge(RouteEdge):
 
 LNPaymentPath = Sequence[PathEdge]
 LNPaymentRoute = Sequence[RouteEdge]
+LNPaymentTRoute = Sequence[TrampolineEdge]
 
 
-def is_route_sane_to_use(route: LNPaymentRoute, invoice_amount_msat: int, min_final_cltv_expiry: int) -> bool:
+def is_route_within_budget(
+    route: LNPaymentRoute,
+    *,
+    budget: PaymentFeeBudget,
+    amount_msat_for_dest: int,  # that final receiver gets
+    cltv_delta_for_dest: int,   # that final receiver gets
+) -> bool:
     """Run some sanity checks on the whole route, before attempting to use it.
     called when we are paying; so e.g. lower cltv is better
     """
     if len(route) > NUM_MAX_EDGES_IN_PAYMENT_PATH:
         return False
-    amt = invoice_amount_msat
-    cltv = min_final_cltv_expiry
+    amt = amount_msat_for_dest
+    cltv_cost_of_route = 0  # excluding cltv_delta_for_dest
     for route_edge in reversed(route[1:]):
-        if not route_edge.is_sane_to_use(amt): return False
         amt += route_edge.fee_for_edge(amt)
-        cltv += route_edge.cltv_expiry_delta
-    total_fee = amt - invoice_amount_msat
-    # TODO revise ad-hoc heuristics
-    if cltv > NBLOCK_CLTV_EXPIRY_TOO_FAR_INTO_FUTURE:
+        cltv_cost_of_route += route_edge.cltv_delta
+    fee_cost = amt - amount_msat_for_dest
+    # check against budget
+    if cltv_cost_of_route > budget.cltv:
         return False
-    if not is_fee_sane(total_fee, payment_amount_msat=invoice_amount_msat):
+    if fee_cost > budget.fee_msat:
+        return False
+    # sanity check
+    total_cltv_delta = cltv_cost_of_route + cltv_delta_for_dest
+    if total_cltv_delta > NBLOCK_CLTV_DELTA_TOO_FAR_INTO_FUTURE:
         return False
     return True
 
 
-def is_fee_sane(fee_msat: int, *, payment_amount_msat: int) -> bool:
-    # fees <= 5 sat are fine
-    if fee_msat <= 5_000:
-        return True
-    # fees <= 1 % of payment are fine
-    if 100 * fee_msat <= payment_amount_msat:
-        return True
-    return False
-
-
 class LiquidityHint:
     """Encodes the amounts that can and cannot be sent over the direction of a
-    channel and whether the channel is blacklisted.
+    channel.
 
     A LiquidityHint is the value of a dict, which is keyed to node ids and the
     channel.
@@ -181,7 +172,6 @@ class LiquidityHint:
         self._cannot_send_forward = None
         self._can_send_backward = None
         self._cannot_send_backward = None
-        self.blacklist_timestamp = 0
         self.hint_timestamp = 0
         self._inflight_htlcs_forward = 0
         self._inflight_htlcs_backward = 0
@@ -294,10 +284,8 @@ class LiquidityHint:
             self._inflight_htlcs_backward = max(0, self._inflight_htlcs_forward - 1)
 
     def __repr__(self):
-        is_blacklisted = False if not self.blacklist_timestamp else int(time.time()) - self.blacklist_timestamp < BLACKLIST_DURATION
         return f"forward: can send: {self._can_send_forward} msat, cannot send: {self._cannot_send_forward} msat, htlcs: {self._inflight_htlcs_forward}\n" \
-               f"backward: can send: {self._can_send_backward} msat, cannot send: {self._cannot_send_backward} msat, htlcs: {self._inflight_htlcs_backward}\n" \
-               f"blacklisted: {is_blacklisted}"
+               f"backward: can send: {self._can_send_backward} msat, cannot send: {self._cannot_send_backward} msat, htlcs: {self._inflight_htlcs_backward}\n"
 
 
 class LiquidityHintMgr:
@@ -379,22 +367,6 @@ class LiquidityHintMgr:
         inflight_htlc_fee = num_inflight_htlcs * success_fee
         return success_fee + inflight_htlc_fee
 
-    @with_lock
-    def add_to_blacklist(self, channel_id: ShortChannelID):
-        hint = self.get_hint(channel_id)
-        now = int(time.time())
-        hint.blacklist_timestamp = now
-
-    @with_lock
-    def get_blacklist(self) -> Set[ShortChannelID]:
-        now = int(time.time())
-        return set(k for k, v in self._liquidity_hints.items() if now - v.blacklist_timestamp < BLACKLIST_DURATION)
-
-    @with_lock
-    def clear_blacklist(self):
-        for k, v in self._liquidity_hints.items():
-            v.blacklist_timestamp = 0
-
     @with_lock
     def reset_liquidity_hints(self):
         for k, v in self._liquidity_hints.items():
@@ -414,6 +386,34 @@ class LNPathFinder(Logger):
         Logger.__init__(self)
         self.channel_db = channel_db
         self.liquidity_hints = LiquidityHintMgr()
+        self._edge_blacklist = dict()  # type: Dict[ShortChannelID, int]  # scid -> expiration
+        self._blacklist_lock = threading.Lock()
+
+    def _is_edge_blacklisted(self, short_channel_id: ShortChannelID, *, now: int) -> bool:
+        blacklist_expiration = self._edge_blacklist.get(short_channel_id)
+        if blacklist_expiration is None:
+            return False
+        if blacklist_expiration < now:
+            return False
+            # TODO rm expired entries from cache (note: perf vs thread-safety)
+        return True
+
+    def add_edge_to_blacklist(
+        self,
+        short_channel_id: ShortChannelID,
+        *,
+        now: int = None,
+        duration: int = 3600,  # seconds
+    ) -> None:
+        if now is None:
+            now = int(time.time())
+        with self._blacklist_lock:
+            blacklist_expiration = self._edge_blacklist.get(short_channel_id, 0)
+            self._edge_blacklist[short_channel_id] = max(blacklist_expiration, now + duration)
+
+    def clear_blacklist(self):
+        with self._blacklist_lock:
+            self._edge_blacklist = dict()
 
     def update_liquidity_hints(
             self,
@@ -433,6 +433,8 @@ class LNPathFinder(Logger):
                 self.logger.info(f"report {r.short_channel_id} to be unable to forward {amount_msat} msat")
                 self.liquidity_hints.update_cannot_send(r.start_node, r.end_node, r.short_channel_id, amount_msat)
                 break
+        else:
+            assert failing_channel is None
 
     def update_inflight_htlcs(self, route: LNPaymentRoute, add_htlcs: bool):
         self.logger.info(f"{'Adding' if add_htlcs else 'Removing'} inflight htlcs to graph (liquidity hints).")
@@ -445,7 +447,7 @@ class LNPathFinder(Logger):
     def _edge_cost(
             self,
             *,
-            short_channel_id: bytes,
+            short_channel_id: ShortChannelID,
             start_node: bytes,
             end_node: bytes,
             payment_amt_msat: int,
@@ -453,10 +455,13 @@ class LNPathFinder(Logger):
             is_mine=False,
             my_channels: Dict[ShortChannelID, 'Channel'] = None,
             private_route_edges: Dict[ShortChannelID, RouteEdge] = None,
+            now: int,  # unix ts
     ) -> Tuple[float, int]:
         """Heuristic cost (distance metric) of going through a channel.
         Returns (heuristic_cost, fee_for_edge_msat).
         """
+        if self._is_edge_blacklisted(short_channel_id, now=now):
+            return float('inf'), 0
         if private_route_edges is None:
             private_route_edges = {}
         channel_info = self.channel_db.get_channel_info(
@@ -464,12 +469,12 @@ class LNPathFinder(Logger):
         if channel_info is None:
             return float('inf'), 0
         channel_policy = self.channel_db.get_policy_for_node(
-            short_channel_id, start_node, my_channels=my_channels, private_route_edges=private_route_edges)
+            short_channel_id, start_node, my_channels=my_channels, private_route_edges=private_route_edges, now=now)
         if channel_policy is None:
             return float('inf'), 0
         # channels that did not publish both policies often return temporary channel failure
         channel_policy_backwards = self.channel_db.get_policy_for_node(
-            short_channel_id, end_node, my_channels=my_channels, private_route_edges=private_route_edges)
+            short_channel_id, end_node, my_channels=my_channels, private_route_edges=private_route_edges, now=now)
         if (channel_policy_backwards is None
                 and not is_mine
                 and short_channel_id not in private_route_edges):
@@ -487,14 +492,21 @@ class LNPathFinder(Logger):
         route_edge = private_route_edges.get(short_channel_id, None)
         if route_edge is None:
             node_info = self.channel_db.get_node_info_for_node_id(node_id=end_node)
+            if node_info:
+                # it's ok if we are missing the node_announcement (node_info) for this node,
+                # but if we have it, we enforce that they support var_onion_optin
+                node_features = LnFeatures(node_info.features)
+                if not node_features.supports(LnFeatures.VAR_ONION_OPT):  # note: this is kind of slow. could be cached.
+                    return float('inf'), 0
             route_edge = RouteEdge.from_channel_policy(
                 channel_policy=channel_policy,
                 short_channel_id=short_channel_id,
                 start_node=start_node,
                 end_node=end_node,
                 node_info=node_info)
-        if not route_edge.is_sane_to_use(payment_amt_msat):
-            return float('inf'), 0  # thanks but no thanks
+        # Cap cltv of any given edge at 2 weeks (the cost function would not work well for extreme cases)
+        if route_edge.cltv_delta > 14 * 144:
+            return float('inf'), 0
         # Distance metric notes:  # TODO constants are ad-hoc
         # ( somewhat based on https://github.com/lightningnetwork/lnd/pull/1358 )
         # - Edges have a base cost. (more edges -> less likely none will fail)
@@ -504,7 +516,7 @@ class LNPathFinder(Logger):
         if ignore_costs:
             return DEFAULT_PENALTY_BASE_MSAT, 0
         fee_msat = route_edge.fee_for_edge(payment_amt_msat)
-        cltv_cost = route_edge.cltv_expiry_delta * payment_amt_msat * 15 / 1_000_000_000
+        cltv_cost = route_edge.cltv_delta * payment_amt_msat * 15 / 1_000_000_000
         # the liquidty penalty takes care we favor edges that should be able to forward
         # the payment and penalize edges that cannot
         liquidity_penalty = self.liquidity_hints.penalty(start_node, end_node, short_channel_id, payment_amt_msat)
@@ -514,24 +526,31 @@ class LNPathFinder(Logger):
     def get_shortest_path_hops(
             self,
             *,
-            nodeA: bytes,
+            nodeA: bytes, # nodeA is expected to be our node id if channels are passed in my_sending_channels
             nodeB: bytes,
             invoice_amount_msat: int,
             my_sending_channels: Dict[ShortChannelID, 'Channel'] = None,
             private_route_edges: Dict[ShortChannelID, RouteEdge] = None,
+            node_filter: Optional[Callable[[bytes, NodeInfo], bool]] = None
     ) -> Dict[bytes, PathEdge]:
         # note: we don't lock self.channel_db, so while the path finding runs,
         #       the underlying graph could potentially change... (not good but maybe ~OK?)
 
+        # if destination is filtered, there is no route
+        if node_filter:
+            node_info = self.channel_db.get_node_info_for_node_id(nodeB)
+            if not node_filter(nodeB, node_info):
+                return {}
+
         # run Dijkstra
         # The search is run in the REVERSE direction, from nodeB to nodeA,
         # to properly calculate compound routing fees.
-        blacklist = self.liquidity_hints.get_blacklist()
         distance_from_start = defaultdict(lambda: float('inf'))
         distance_from_start[nodeB] = 0
         previous_hops = {}  # type: Dict[bytes, PathEdge]
         nodes_to_explore = queue.PriorityQueue()
         nodes_to_explore.put((0, invoice_amount_msat, nodeB))  # order of fields (in tuple) matters!
+        now = int(time.time())
 
         # main loop of search
         while nodes_to_explore.qsize() > 0:
@@ -558,18 +577,23 @@ class LNPathFinder(Logger):
 
             for edge_channel_id in channels_for_endnode:
                 assert isinstance(edge_channel_id, bytes)
-                if blacklist and edge_channel_id in blacklist:
+                if self._is_edge_blacklisted(edge_channel_id, now=now):
                     continue
                 channel_info = self.channel_db.get_channel_info(
                     edge_channel_id, my_channels=my_sending_channels, private_route_edges=private_route_edges)
                 if channel_info is None:
                     continue
                 edge_startnode = channel_info.node2_id if channel_info.node1_id == edge_endnode else channel_info.node1_id
+                if node_filter:
+                    node_info = self.channel_db.get_node_info_for_node_id(edge_startnode)
+                    if not node_filter(edge_startnode, node_info):
+                        continue
                 is_mine = edge_channel_id in my_sending_channels
-                if is_mine:
-                    if edge_startnode == nodeA:  # payment outgoing, on our channel
-                        if not my_sending_channels[edge_channel_id].can_pay(amount_msat, check_frozen=True):
-                            continue
+                if edge_startnode == nodeA and my_sending_channels:  # payment outgoing, on our channel
+                    if edge_channel_id not in my_sending_channels:
+                        continue
+                    if not my_sending_channels[edge_channel_id].can_pay(amount_msat, check_frozen=True):
+                        continue
                 edge_cost, fee_for_edge_msat = self._edge_cost(
                     short_channel_id=edge_channel_id,
                     start_node=edge_startnode,
@@ -578,7 +602,9 @@ class LNPathFinder(Logger):
                     ignore_costs=(edge_startnode == nodeA),
                     is_mine=is_mine,
                     my_channels=my_sending_channels,
-                    private_route_edges=private_route_edges)
+                    private_route_edges=private_route_edges,
+                    now=now,
+                )
                 alt_dist_to_neighbour = distance_from_start[edge_endnode] + edge_cost
                 if alt_dist_to_neighbour < distance_from_start[edge_startnode]:
                     distance_from_start[edge_startnode] = alt_dist_to_neighbour
@@ -603,6 +629,7 @@ class LNPathFinder(Logger):
             invoice_amount_msat: int,
             my_sending_channels: Dict[ShortChannelID, 'Channel'] = None,
             private_route_edges: Dict[ShortChannelID, RouteEdge] = None,
+            node_filter: Optional[Callable[[bytes, NodeInfo], bool]] = None
     ) -> Optional[LNPaymentPath]:
         """Return a path from nodeA to nodeB."""
         assert type(nodeA) is bytes
@@ -616,7 +643,8 @@ class LNPathFinder(Logger):
             nodeB=nodeB,
             invoice_amount_msat=invoice_amount_msat,
             my_sending_channels=my_sending_channels,
-            private_route_edges=private_route_edges)
+            private_route_edges=private_route_edges,
+            node_filter=node_filter)
 
         if nodeA not in previous_hops:
             return None  # no path found
@@ -676,7 +704,7 @@ class LNPathFinder(Logger):
             nodeA: bytes,
             nodeB: bytes,
             invoice_amount_msat: int,
-            path = None,
+            path: Optional[Sequence[PathEdge]] = None,
             my_sending_channels: Dict[ShortChannelID, 'Channel'] = None,
             private_route_edges: Dict[ShortChannelID, RouteEdge] = None,
     ) -> Optional[LNPaymentRoute]:
